// ignore: implementation_imports
import 'package:analyzer/src/dart/element/type.dart';
import 'package:analyzer/dart/element/type.dart';
import 'package:dao_wrapped_annotation/dao_wrapped_annotation.dart';

class CreateTable {
  String create(InterfaceTypeImpl model, String primaryKey) {
    final fields = <Field>[];

    model.accessors.forEach((v) {
      if (v.type.parameters.isEmpty) {
        fields.add(
          Field(
            v.displayName,
            v.type.returnType,
          ),
        );
      }
    });
    String currentPrimaryKey;

    final sqlFields = <SqlField>[];
    fields.forEach((field) {
      bool isPrimary = false;
      if (field.name == primaryKey) {
        isPrimary = true;
        currentPrimaryKey = field.name;
      }

      sqlFields.add(
        SqlField(_typeToSql(field.type), field.name, isPrimary, false),
      );
    });

    if (currentPrimaryKey == null && sqlFields.isNotEmpty) {
      if (primaryKey != null) {
        throw "invalid primaryKey: $primaryKey";
      }
      sqlFields.first.isPrimary = true;
      currentPrimaryKey = sqlFields.first.name;
    }
    var tableOut = "";

    tableOut += "class _Table extends ModelTable { ";
    tableOut += "@override ";
    tableOut += "List<SqlField> fields; ";
    tableOut += "@override ";
    tableOut += "String name = \"$model\";\n ";
    tableOut += "@override ";
    tableOut += "String primaryKey = \"$currentPrimaryKey\";\n ";

    for (SqlField field in sqlFields) {
      tableOut += "final $SqlField f${_firstCharToUpperCase(field.name)} = "
          "$SqlField(${field.type},\"${field.name}\",${field.isPrimary},${field.autoIncrement},); ";
    }
    tableOut +=
        "_Table() {fields = ${sqlFields.map((item) => "f" + _firstCharToUpperCase(item.name)).toList()};}";
    tableOut += "}";
    return tableOut;
  }

  String _firstCharToUpperCase(String string) {
    if (string.isEmpty) {
      return string;
    }
    return string.replaceRange(0, 1, string[0].toUpperCase());
  }

  _typeToSql(DartType type) {
    if (type.isDartCoreInt) {
      return SqlType.INTEGER;
    } else if (type.isDartCoreDouble) {
      return SqlType.REAL;
    } else if (type.isDartCoreBool) {
      return SqlType.BLOB;
    } else {
      return SqlType.TEXT;
    }
  }
}

class Field {
  final String name;
  final DartType type;

  Field(this.name, this.type);
}
